# 前準備

aws cliをインストールし、AWSの管理者でアクセスできるようにする。
また、EC2インスタンスにsshでアクセスするので、sshも必要となる。

# DBパスワードを格納する。
```
$ cat > dbpass.txt
xxxxxxx
^D
$ ProjectName=wagby
$ aws ssm put-parameter --name /application/${ProjectName}/DBMasterUserPassword \
 --type SecureString --value $(cat dbpass.txt)
$ rm dbpass.txt
```

格納したパスワードは下記で確認できる。
```
$ aws ssm get-parameter --name /application/${ProjectName}/DBMasterUserPassword  --with-decryption
{
    "Parameter": {
        "Name": "/application/wagby/DBMasterUserPassword",
        "Type": "SecureString",
        "Value": "xxxxxxx",
        "Version": 1,
        "LastModifiedDate": "2021-07-20T07:58:23.568000+01:00",
        "ARN": "arn:aws:ssm:ap-northeast-1:140725439745:parameter/application/wagby/DBMasterUserPassword",
        "DataType": "text"
    }
}
```

# sshアクセスのためのキーペアを作成する。
```
$ KeyPairName=pgtestenv
$ aws ec2 create-key-pair --key-name ${KeyPairName}
{
    "KeyFingerprint": "52:37:4d:8d:9b:09:f4:57:95:98:57:9f:45:0a:7f:3a:3d:9f:ec:b5",
    "KeyMaterial": "-----BEGIN RSA PRIVATE KEY-----\n...=\n-----END RSA PRIVATE KEY-----",
    "KeyName": "pgtestenv",
    "KeyPairId": "key-0b86c9607dafd2478"
}
```

KeyMaterialの部分をコピー&ペーストでsshのキーとして保存する。ウェブ画面のAWS Management Consoleからでもできる。
```
$ SSHKeyFilename=~/users/.ssh/aws_pgtestenv
$ cat > ${SSHKeyFilename}.txt
[上記のKeyMaterial部分をコピー&ペースト]
^D
$ cat ${SSHKeyFilename}.txt | tr -d '\n' | sed 's/\\n/\n/g' > ${SSHKeyFilename}
$ chmod 600 ${SSHKeyFilename}
$ ssh-keygen -yf ${SSHKeyFilename} > ${SSHKeyFilename}.pub
```

環境を作成する。
```
aws cloudformation deploy --stack-name pgtestenv-${ProjectName} \
--template-file create-pgtestenv.json \
--capabilities CAPABILITY_NAMED_IAM \
--parameter-overrides ProjectName=${ProjectName} KeyPair=${KeyPairName}
```

# 作成した環境のホスト名を取得
```
$ aws cloudformation describe-stacks --stack-name pgtestenv-wagby --query "Stacks[0].Outputs"
[
    {
        "OutputKey": "RDSEndpointPort",
        "OutputValue": "5432",
        "Description": "RDS Endpoint port"
    },
    {
        "OutputKey": "RDSEndpointAddress",
        "OutputValue": "pgtestenv-wagby.cinfwbelovtg.ap-northeast-1.rds.amazonaws.com",
        "Description": "RDS Endpoint address"
    },
    {
        "OutputKey": "EC2InstanceId",
        "OutputValue": "i-09f25c8974e0e758c",
        "Description": "EC2Instance id"
    },
    {
        "OutputKey": "EC2InstancePublicDnsName",
        "OutputValue": "ec2-13-115-3-91.ap-northeast-1.compute.amazonaws.com",
        "Description": "EC2Instance public hostname"
    }
]
$ export EC2Hostname=$(aws cloudformation describe-stacks --stack-name pgtestenv-wagby --query "Stacks[0].Outputs[?OutputKey=='EC2InstancePublicDnsName'].OutputValue" --output text | sed 's/\r//g')
$ echo $EC2Hostname
ec2-13-115-3-91.ap-northeast-1.compute.amazonaws.com
```

# sshログオンする。

```
$ ssh -i ${SSHKeyFilename} ec2-user@${EC2Hostname}

       __|  __|_  )
       _|  (     /   Amazon Linux 2 AMI
      ___|\___|___|

https://aws.amazon.com/amazon-linux-2/
[ec2-user@ip-10-0-1-176 ~]$ java -version
openjdk version "1.8.0_282"
OpenJDK Runtime Environment (build 1.8.0_282-b08)
OpenJDK 64-Bit Server VM (build 25.282-b08, mixed mode)
[ec2-user@ip-10-0-1-176 ~]$ psql --version
psql (PostgreSQL) 9.2.24
```

# psqlコマンドにて作成したデータベースにSSL接続を行う。

SSL接続はできたが、証明書の検証は行われていないことがわかる。
```
[ec2-user@ip-10-0-1-176 ~]$ export PGHOST=pgtestenv-wagby.cinfwbelovtg.ap-northeast-1.rds.amazonaws.com
[ec2-user@ip-10-0-1-176 ~]$ export PGUSER=wagby
[ec2-user@ip-10-0-1-176 ~]$ export PGDATABASE=wagbydb
[ec2-user@ip-10-0-1-176 ~]$ cat > .pgpass
pgtestenv-wagby.cinfwbelovtg.ap-northeast-1.rds.amazonaws.com:5432:wagbydb:wagby:xxxxxxxx
^D
[ec2-user@ip-10-0-1-176 ~]$ chmod 600 ~/.pgpass
[ec2-user@ip-10-0-1-176 ~]$ psql
psql (9.2.24, server 12.7)
WARNING: psql version 9.2, server version 12.0.
         Some psql features might not work.
SSL connection (cipher: ECDHE-RSA-AES256-GCM-SHA384, bits: 256)
Type "help" for help.

wagbydb=> select datname, usename, ssl, version, cipher, bits, compression, client_addr from pg_stat_ssl inner join pg_stat_activity on pg_stat_ssl.pid = pg_stat_activity.pid where ssl is true and usename<>'rdsadmin';
 datname | usename | ssl | version |           cipher            | bits | compression | client_addr
---------+---------+-----+---------+-----------------------------+------+-------------+-------------
 wagbydb | wagby   | t   | TLSv1.2 | ECDHE-RSA-AES256-GCM-SHA384 |  256 | f           | 10.0.1.176
(1 row)

wagbydb=> \q
[ec2-user@ip-10-0-1-176 ~]$ psql "sslmode=disable"
psql (9.2.24, server 12.7)
WARNING: psql version 9.2, server version 12.0.
         Some psql features might not work.
Type "help" for help.

wagbydb=> select datname, usename, ssl, version, cipher, bits, compression, client_addr from pg_stat_ssl inner join pg_stat_activity on pg_stat_ssl.pid = pg_stat_activity.pid where ssl is true and usename<>'rdsadmin';
 datname | usename | ssl | version | cipher | bits | compression | client_addr
---------+---------+-----+---------+--------+------+-------------+-------------
(0 rows)

wagbydb=> \q
[ec2-user@ip-10-0-1-176 ~]$ psql "sslmode=require"
psql (9.2.24, server 12.7)
WARNING: psql version 9.2, server version 12.0.
         Some psql features might not work.
SSL connection (cipher: ECDHE-RSA-AES256-GCM-SHA384, bits: 256)
Type "help" for help.

wagbydb=> select datname, usename, ssl, version, cipher, bits, compression, client_addr from pg_stat_ssl inner join pg_stat_activity on pg_stat_ssl.pid = pg_stat_activity.pid where ssl is true and usename<>'rdsadmin';
 datname | usename | ssl | version |           cipher            | bits | compression | client_addr
---------+---------+-----+---------+-----------------------------+------+-------------+-------------
 wagbydb | wagby   | t   | TLSv1.2 | ECDHE-RSA-AES256-GCM-SHA384 |  256 | f           | 10.0.1.176
(1 row)

wagbydb=> \q
[ec2-user@ip-10-0-1-176 ~]$ psql "sslmode=verify-ca"
psql: root certificate file "/home/ec2-user/.postgresql/root.crt" does not exist
Either provide the file or change sslmode to disable server certificate verification.
[ec2-user@ip-10-0-1-176 ~]$ psql "sslmode=verify-full"
psql: root certificate file "/home/ec2-user/.postgresql/root.crt" does not exist
Either provide the file or change sslmode to disable server certificate verification.
```

# 証明書の検証を行って、接続する。

SSL/TLS を使用した DB インスタンスへの接続の暗号化
https://docs.aws.amazon.com/ja_jp/AmazonRDS/latest/UserGuide/UsingWithRDS.SSL.html

PostgreSQL DB インスタンスで SSL を使用する
https://docs.aws.amazon.com/ja_jp/AmazonRDS/latest/UserGuide/PostgreSQL.Concepts.General.SSL.html

すべての AWS リージョンの中間証明書とルート証明書の両方を含む証明書バンドルを使用する。
```
[ec2-user@ip-10-0-1-176 ~]$ curl https://truststore.pki.rds.amazonaws.com/global/global-bundle.pem > global-bundle.pem
  % Total    % Received % Xferd  Average Speed   Time    Time     Time  Current
                                 Dload  Upload   Total   Spent    Left  Speed
100  146k  100  146k    0     0   245k      0 --:--:-- --:--:-- --:--:--  245k
[ec2-user@ip-10-0-1-176 ~]$ psql "sslrootcert=global-bundle.pem sslmode=verify-full"
psql (9.2.24, server 12.7)
WARNING: psql version 9.2, server version 12.0.
         Some psql features might not work.
SSL connection (cipher: ECDHE-RSA-AES256-GCM-SHA384, bits: 256)
Type "help" for help.

wagbydb=> \q
[ec2-user@ip-10-0-1-176 ~]$ psql "sslrootcert=global-bundle.pem sslmode=verify-ca"
psql (9.2.24, server 12.7)
WARNING: psql version 9.2, server version 12.0.
         Some psql features might not work.
SSL connection (cipher: ECDHE-RSA-AES256-GCM-SHA384, bits: 256)
Type "help" for help.

wagbydb=> \q
```

アジアパシフィック (東京)リージョンの証明書バンドルを使用する。
```
[ec2-user@ip-10-0-1-176 ~]$ curl https://truststore.pki.rds.amazonaws.com/ap-northeast-1/ap-northeast-1-bundle.pem > ap-northeast-1-bundle.pem
  % Total    % Received % Xferd  Average Speed   Time    Time     Time  Current
                                 Dload  Upload   Total   Spent    Left  Speed
100  7540  100  7540    0     0  16076      0 --:--:-- --:--:-- --:--:-- 16076
[ec2-user@ip-10-0-1-176 ~]$ psql "sslrootcert=ap-northeast-1-bundle.pem sslmode=verify-ca"
psql (9.2.24, server 12.7)
WARNING: psql version 9.2, server version 12.0.
         Some psql features might not work.
SSL connection (cipher: ECDHE-RSA-AES256-GCM-SHA384, bits: 256)
Type "help" for help.

wagbydb=> \q
[ec2-user@ip-10-0-1-176 ~]$ psql "sslrootcert=ap-northeast-1-bundle.pem sslmode=verify-full"
psql (9.2.24, server 12.7)
WARNING: psql version 9.2, server version 12.0.
         Some psql features might not work.
SSL connection (cipher: ECDHE-RSA-AES256-GCM-SHA384, bits: 256)
Type "help" for help.

wagbydb=> \q
```

# wagbyappを作成し、EC2インスタンスに転送する。

WagbyDesigner内に下記ファイルを格納する。

customize/tomcat/conf フォルダを作成し、customize/tomcat/conf/aws-rds-global-bundle.pemにAWS RDSの証明書 global-bundle.pem を格納する。

customize/tomcat/lib/postgresql-42.2.23.jarにPostgreSQLのJDBCドライバを格納する。

WagbyDesignerを起動し、定義を行う。
環境 - データベース - メイン - 接続 に、下記の設定を行う。
データベースに PostgreSQL 9.4 or later を指定する。
ユーザ名、パスワードに wagby を指定する。

設定後、フルビルドする。

フルビルド後、証明書がwagbyapp/confにあることを確認したうえで、EC2インスタンスに転送する。
```
$ ls -l wagbyapp/conf/aws-rds-global-bundle.pem
-rwxr-xr-x 1 mura mura 149520 7月  20 17:50 wagbyapp/conf/aws-rds-global-bundle.pem
$ tar -czf wagbyapp.tar.gz wagbyapp/
$ scp -i $SSHKeyFilename wagbyapp.tar.gz ec2-user@${EC2Hostname}:
```

https://jdbc.postgresql.org/documentation/head/connect.html#connection-parameters

# check_db.shにて接続を確認する。

証明書の検証を行わず、SSLで接続できるのを確認した。
```
[ec2-user@ip-10-0-1-176 ~]$ ls
ap-northeast-1-bundle.pem  global-bundle.pem  wagbyapp.tar.gz
[ec2-user@ip-10-0-1-176 ~]$ tar -xvzf wagbyapp.tar.gz
[ec2-user@ip-10-0-1-176 ~]$ cd wagbyapp/bin/
[ec2-user@ip-10-0-1-176 bin]$ cp ../webapps/wagby/WEB-INF/export/conf/initdb.xml{,.orig}
[ec2-user@ip-10-0-1-176 bin]$ vi ../webapps/wagby/WEB-INF/export/conf/initdb.xml
[ec2-user@ip-10-0-1-176 bin]$ diff ../webapps/wagby/WEB-INF/export/conf/initdb.xml{.orig,}
9c9
<                  url="jdbc:postgresql://localhost/wagbydb"
---
>                  url="jdbc:postgresql://pgtestenv-wagby.cinfwbelovtg.ap-northeast-1.rds.amazonaws.com/wagbydb?ssl=true&amp;sslmode=require"
11c11
<             password="wagby"
---
>             password="xxxxxxxx"
[ec2-user@ip-10-0-1-176 bin]$ ./check_db.sh
[ec2-user@ip-10-0-1-176 bin]$ head check_db.txt
Command: driver_all
All table enabled
Config File:../webapps/wagby/WEB-INF/export/conf/initdb.xml
Log4j Property File: default file.
allProceduresAreCallable=true
allTablesAreSelectable=true
dataDefinitionCausesTransactionCommit=false
dataDefinitionIgnoredInTransactions=false
deletesAreDetected(ResultSet.TYPE_FORWARD_ONLY)=false
deletesAreDetected(ResultSet.TYPE_SCROLL_INSENSITIVE)=false
```

sslmode=verify-fullとして証明書の検証を行うようにしたが、root caを指定していないため、接続に失敗する。
```
[ec2-user@ip-10-0-1-176 bin]$ vi ../webapps/wagby/WEB-INF/export/conf/initdb.xml
[ec2-user@ip-10-0-1-176 bin]$ diff ../webapps/wagby/WEB-INF/export/conf/initdb.xml{.orig,}
9c9
<                  url="jdbc:postgresql://localhost/wagbydb"
---
>         url="jdbc:postgresql://pgtestenv-wagby.cinfwbelovtg.ap-northeast-1.rds.amazonaws.com/wagbydb?ssl=true&amp;sslmode=verify-full"
11c11
<             password="wagby"
---
>             password="xxxxxxxx"
[ec2-user@ip-10-0-1-176 bin]$ ./check_db.sh
[ec2-user@ip-10-0-1-176 bin]$ head check_db.txt
Command: driver_all
All table enabled
Config File:../webapps/wagby/WEB-INF/export/conf/initdb.xml
Log4j Property File: default file.
2021-07-20 09:15:04 [ERROR jp.jasminesoft.jfc.initdb.XML2DBLoader] (admin@localhost) JFC-00113:Falied to create connection.
org.apache.commons.dbcp.SQLNestedException: Cannot create PoolableConnectionFactory (Could not open SSL root certificate file /home/ec2-user/.postgresql/root.crt.)
        at org.apache.commons.dbcp.BasicDataSource.createPoolableConnectionFactory(BasicDataSource.java:1549)
        at org.apache.commons.dbcp.BasicDataSource.createDataSource(BasicDataSource.java:1388)
        at org.apache.commons.dbcp.BasicDataSource.getConnection(BasicDataSource.java:1044)
        at jp.jasminesoft.jfc.initdb.XML2DBLoader.db_init(XML2DBLoader.java:392)
```

証明書のファイルパスを設定することで、接続に成功することを確認した。
```
[ec2-user@ip-10-0-1-176 bin]$ ls ../conf/aws-rds-global-bundle.pem
../conf/aws-rds-global-bundle.pem
[ec2-user@ip-10-0-1-176 bin]$ vi ../webapps/wagby/WEB-INF/export/conf/initdb.xml
[ec2-user@ip-10-0-1-176 bin]$ diff ../webapps/wagby/WEB-INF/export/conf/initdb.xml{.orig,}
9c9
<                  url="jdbc:postgresql://localhost/wagbydb"
---
>         url="jdbc:postgresql://pgtestenv-wagby.cinfwbelovtg.ap-northeast-1.rds.amazonaws.com/wagbydb?ssl=true&amp;sslmode=verify-full&amp;sslrootcert=${catalina.home}/conf/aws-rds-global-bundle.pem"
11c11
<             password="wagby"
---
>             password="xxxxxxxx"
[ec2-user@ip-10-0-1-176 bin]$ ./check_db.sh
[ec2-user@ip-10-0-1-176 bin]$ head check_db.txt
Command: driver_all
All table enabled
Config File:../webapps/wagby/WEB-INF/export/conf/initdb.xml
Log4j Property File: default file.
allProceduresAreCallable=true
allTablesAreSelectable=true
dataDefinitionCausesTransactionCommit=false
dataDefinitionIgnoredInTransactions=false
deletesAreDetected(ResultSet.TYPE_FORWARD_ONLY)=false
deletesAreDetected(ResultSet.TYPE_SCROLL_INSENSITIVE)=false
```

# init_db.shを実行する。

```
[ec2-user@ip-10-0-1-176 bin]$ ./init_db.sh
Command: auto
All table enabled
Config File:../webapps/wagby/WEB-INF/export/conf/initdb.xml
Export Directory Path:../webapps/wagby/WEB-INF/export
Log4j Property File: default file.
import hourM 24/24 at 0 sec. process 24 at 0 sec. rest time: 0 or 0 sec.
import minuteM 6/6 at 0 sec. process 6 at 0 sec. rest time: 0 or 0 sec.
import movestepM 12/12 at 0 sec. process 12 at 0 sec. rest time: 0 or 0 sec.
import jcategory 7/7 at 0 sec. process 7 at 0 sec. rest time: 0 or 0 sec.
import jprincipal 30/30 at 0 sec. process 30 at 0 sec. rest time: 0 or 0 sec.
import juser 2/2 at 0 sec. process 2 at 0 sec. rest time: 0 or 0 sec.
import jfcjob 16/16 at 0 sec. process 16 at 0 sec. rest time: 0 or 0 sec.
import jfcmodel 65/65 at 0 sec. process 65 at 0 sec. rest time: 0 or 0 sec.
import jfcflow_setting 11/11 at 0 sec. process 11 at 0 sec. rest time: 0 or 0 sec.
import jfcmodel4dm 1/1 at 0 sec. process 1 at 0 sec. rest time: 0 or 0 sec.
import jfcportal 1/1 at 0 sec. process 1 at 0 sec. rest time: 0 or 0 sec.
import jfcportlet 5/5 at 0 sec. process 5 at 0 sec. rest time: 0 or 0 sec.
import jfcportletGraphColor_m 6/6 at 0 sec. process 6 at 0 sec. rest time: 0 or 0 sec.
import jfcportletGraphType_m 12/12 at 0 sec. process 12 at 0 sec. rest time: 0 or 0 sec.
```

# wagbyappを起動する。

TLS1.2で接続できていることも確認できる。

新しい SSL/TLS 証明書を使用して Aurora PostgreSQL DB クラスターに接続するようにアプリケーションを更新する
https://docs.aws.amazon.com/ja_jp/AmazonRDS/latest/AuroraUserGuide/ssl-certificate-rotation-aurora-postgresql.html

```
[ec2-user@ip-10-0-1-176 bin]$ cp ../webapps/wagby/META-INF/context.xml  ../webapps/wagby/META-INF/context.xml.orig
[ec2-user@ip-10-0-1-176 bin]$ vi ../webapps/wagby/META-INF/context.xml
[ec2-user@ip-10-0-1-176 bin]$ diff ../webapps/wagby/META-INF/context.xml{.orig,}
20c20
<             url="jdbc:postgresql://localhost/wagbydb"
---
>             url="jdbc:postgresql://pgtestenv-wagby.cinfwbelovtg.ap-northeast-1.rds.amazonaws.com/wagbydb?ssl=true&amp;sslmode=verify-full&amp;sslrootcert=${catalina.home}/conf/aws-rds-global-bundle.pem"
22c22
<             password="wagby"
---
>             password="xxxxxxxx"
[ec2-user@ip-10-0-1-176 bin]$ ./startup.sh
Using CATALINA_BASE:   /home/ec2-user/wagbyapp
Using CATALINA_HOME:   /home/ec2-user/wagbyapp
Using CATALINA_TMPDIR: /home/ec2-user/wagbyapp/temp
Using JRE_HOME:        /usr
Using CLASSPATH:       /home/ec2-user/wagbyapp/bin/bootstrap.jar:/home/ec2-user/wagbyapp/bin/tomcat-juli.jar
Using CATALINA_OPTS:   -server -Xmx512m -Xms64m -Djava.security.auth.login.config=/home/ec2-user/wagbyapp/conf/app_jaas.config  -Dsun.nio.cs.map=x-windows-iso2022jp/ISO-2022-JP -Duser.timezone=Asia/Tokyo
Tomcat started.
[ec2-user@ip-10-0-1-176 bin]$ tail -f ../logs/catalina.out
...
  .   ____          _            __ _ _
 /\\ / ___'_ __ _ _(_)_ __  __ _ \ \ \ \
( ( )\___ | '_ | '_| | '_ \/ _` | \ \ \ \
 \\/  ___)| |_)| | | | | || (_| |  ) ) ) )
  '  |____| .__|_| |_|_| |_\__, | / / / /
 =========|_|==============|___/=/_/_/_/
 :: Spring Boot ::       (v2.2.11.RELEASE)

20-Jul-2021 18:28:36.936 INFO [main] org.apache.catalina.startup.HostConfig.deployDirectory Deployment of web application directory [/home/ec2-user/wagbyapp/webapps/wagby] has finished in [33,903] ms
20-Jul-2021 18:28:36.941 INFO [main] org.apache.coyote.AbstractProtocol.start Starting ProtocolHandler ["http-nio-8921"]
20-Jul-2021 18:28:36.954 INFO [main] org.apache.catalina.startup.Catalina.start Server startup in [33995] milliseconds
[ec2-user@ip-10-0-1-176 bin]$ psql sslmode=disable
psql (9.2.24, server 12.7)
WARNING: psql version 9.2, server version 12.0.
         Some psql features might not work.
Type "help" for help.

wagbydb=> select datname, usename, ssl, version, cipher, bits, compression, client_addr from pg_stat_ssl inner join pg_stat_activity on pg_stat_ssl.pid = pg_stat_activity.pid where ssl is true and usename<>'rdsadmin';
 datname | usename | ssl | version |           cipher            | bits | compression | client_addr
---------+---------+-----+---------+-----------------------------+------+-------------+-------------
 wagbydb | wagby   | t   | TLSv1.2 | ECDHE-RSA-AES256-GCM-SHA384 |  256 | f           | 10.0.1.176
(1 row)

wagbydb=> \q
```

# Wagbyアプリケーションにアクセスし、動作していることを確認する。

http://ec2-13-115-3-91.ap-northeast-1.compute.amazonaws.com:8921/wagby/

# 注意点

RDSのサーバ証明書には有効期限がある。
有効期限前にTomcat側のRDSのサーバ証明書を最新に入れ替え、その後RDS側にて新しい証明書を使うように設定する必要がある。

SSL/TLS 証明書のローテーション
https://docs.aws.amazon.com/ja_jp/AmazonRDS/latest/UserGuide/UsingWithRDS.SSL-certificate-rotation.html

Amazon RDS のお客様: 2020 年 3 月 5 日までに SSL/TLS 証明書を更新してください
https://aws.amazon.com/jp/blogs/news/amazon-rds-customers-update-your-ssl-tls-certificates-by-february-5-2020/

今回起動したRDSインスタンスが使っている証明書はrds-ca-2019なので、2024 年 8 月 22 日が有効期限となるようです。
```
$ aws rds describe-db-instances --query DBInstances[].CACertificateIdentifier
[
    "rds-ca-2019"
]
```
